Imports System.Data.Entity
Imports System.Data.Entity.Infrastructure
Imports CompanyDatabaseInitialization
Imports DataLayer
Imports DomainClasses
Imports Microsoft.VisualStudio.TestTools.UnitTesting

Namespace AutomatedTests
  <TestClass>
  Public Class TestContexts
	Public Sub New()
	  HibernatingRhinos.Profiler.Appender.EntityFramework.EntityFrameworkProfiler.Initialize()
	  Database.SetInitializer(Of ShippingContext)(Nothing)
	End Sub

	  'RUN THIS TEST FIRST TO CREATE THE DATABASE
	  'IT WILL ALSO FORCE THE SEED METHOD WHICH WILL CREATE
	  'SOME DATA AND A NEEDED DATABASE VIEW
	  'SEE THE COMPANYDATABASEMODEL.vb FILE FOR DETAILS
	  <TestMethod>
	  Public Sub InitializeDatabase()
		  Database.SetInitializer(New DemoDBInitializer)
		  Using context = New CompanyDatabase
			  context.Database.Initialize(force:=True)
		  End Using
	  End Sub

	<TestMethod>
	Public Sub CanRetrieveLineItemsToBeShipped()
	  Using context = New ShippingContext
		Debug.WriteLine(context.Database.Connection.ConnectionString)
		Assert.IsTrue(context.ItemsToBeShipped.Any)
	  End Using
	End Sub

	<TestMethod>
	Public Sub GetOrdersWithItemsToBeShipped()
	  Using context = New ShippingContext
		Dim orderQuery = context.Orders.Include(Function(o) o.Customer.ShippingAddress).Where(Function(o) o.OpenLineItems.Any)
		Dim orders = orderQuery.ToList
		Assert.IsTrue(orders.Any)
	  End Using
	End Sub

	<TestMethod>
	Public Sub NewlyShippedItemNotInToBeShippedList()
		Dim unshippedItemId As Integer
		Using context = New ShippingContext
		  Dim unshippedItem = context.ItemsToBeShipped.First
		  unshippedItemId = unshippedItem.LineItemId
		  Dim shipment = New Shipment With {.DateShipped = Date.Now, .EntireOrder = True, .OrderId = unshippedItem.OrderId, .ShipperId = 1}
		  'new shipment won't have shipmentid so need navigation property to do an insert
		  Dim newlyshippedItem = New ShippedLineItem With {.LineItemId = unshippedItemId, .Shipment = shipment}
		  context.Shipments.Add(shipment)
		  context.Entry(newlyshippedItem).State=EntityState.Modified
		  context.SaveChanges()
		End Using
		  Using context = New ShippingContext
			Assert.IsNull(context.ItemsToBeShipped.Find(unshippedItemId))
		  End Using
	End Sub
  End Class
End Namespace
