Imports System.Data.Entity
Imports DataLayer
Imports DataLayer.Mapping
Imports DomainClasses

Namespace CompanyDatabaseInitialization
    Public Class CompanyDatabase
        Inherits DbContext

        Public Sub New()
            MyBase.New("DPSalesDatabase")
        End Sub

        Public Property Customers As DbSet(Of Customer)

        Public Property Employees As DbSet(Of Employee)

        Public Property SalaryHistories As DbSet(Of SalaryHistory)

        Public Property Orders As DbSet(Of Order)

        Public Property LineItems As DbSet(Of LineItem)

        Public Property Products As DbSet(Of Product)

        Public Property Shipments As DbSet(Of Shipment)

        Public Property Shippers As DbSet(Of Shipper)

        Public Property ShippingAddresses As DbSet(Of ShippingAddress)

        Public Property Payments As DbSet(Of Payment)

        Public Property Categories As DbSet(Of Category)

        Public Property Promotions As DbSet(Of Promotion)

        Public Property Returns As DbSet(Of [Return])

        Protected Overrides Sub OnModelCreating(ByVal modelBuilder As DbModelBuilder)
            modelBuilder.Configurations.Add(New LineItemMap)
            modelBuilder.Configurations.Add(New ShippingAddressMap)
            modelBuilder.Entity(Of Product)().Ignore(Function(t) t.LineItems)
        End Sub
    End Class

    Public Class DemoDBInitializer
        Inherits CreateDatabaseIfNotExists(Of CompanyDatabase)

        Protected Overrides Sub Seed(ByVal context As CompanyDatabase)
            Dim biking = New Category With {.Name = "Biking"}
            Dim blackbike = New Product With {.Name = "Black Road Bike", .ProductNumber = "BRB1", .Category = biking, .Color = ProductColor.Black, .SellStartDate = Date.Now}
            Dim redbike = New Product With {.Name = "Red Road Bike", .ProductNumber = "RRB1", .Category = biking, .Color = ProductColor.Red, .SellStartDate = Date.Now}
            Dim julie = New Customer With {.FirstName = "Julie", .LastName = "Lerman", .Orders = New List(Of Order) _
                From {
                New Order With {.OrderDate = Date.Now, .ModifiedDate = Date.Now, .LineItems = New List(Of LineItem) _
                        From {
                New LineItem With {.Product = blackbike, .OrderQty = 1}}}}}
            Dim boris = New Customer With {.FirstName = "boris", .LastName = "Badenov", .Orders = New List(Of Order) _
                From {
                New Order With {.OrderDate = Date.Now, .ModifiedDate = Date.Now, .LineItems = New List(Of LineItem) _
                        From {
                New LineItem With {.Product = redbike, .OrderQty = 1}}}}}

            Dim natasha As Customer = New Customer With {.FirstName = "Natasha", .LastName = "Fatale"}
            context.Customers.Add(julie)
            context.Customers.Add(boris)
            context.Customers.Add(natasha)
            context.Shippers.Add(New Shipper With {.Name = "No Bump Shipping"})
            context.Shippers.Add(New Shipper With {.Name = "From Here to There Shippers"})
            Dim CreateView As String = "CREATE VIEW ItemsToBeShipped " & "AS " & "SELECT L.LineItemId,L.OrderId,L.OrderQty as Quantity,P.Name as ProductName, p.ProductNumber " & "FROM lineitems L,products P  " & "WHERE L.Productid=p.productid and (shipmentid=0 or shipmentid is null)"

            context.Database.ExecuteSqlCommand(CreateView)
            MyBase.Seed(context)
        End Sub
    End Class
End Namespace

